@@ -140,13 +140,14 @@ struct feed {
 void alldirty(void);
 void cleanup(void);
 void draw(void);
-int isurlnew(const char *);
 void markread(struct pane *, off_t, off_t, int);
 void pane_draw(struct pane *);
-void readurls(void);
 void sighandler(int);
 void updategeom(void);
 void updatesidebar(int);
+void urls_free(void);
+int urls_isnew(const char *);
+void urls_read(void);
 
 static struct statusbar statusbar;
 static struct pane panes[PaneLast];
@@ -960,7 +961,17 @@ lineeditor(void)
 		} else if (ch >= ' ') {
 			write(1, &ch, 1);
 		} else if (ch < 0) {
-			continue; /* process signals later */
+			switch (sigstate) {
+			case 0:
+			case SIGWINCH:
+				continue; /* process signals later */
+			case SIGINT:
+				sigstate = 0; /* exit prompt, do not quit */
+			case SIGTERM:
+				break; /* exit prompt and quit */
+			}
+			free(input);
+			return NULL;
 		}
 		input[nchars++] = ch;
 	}
@@ -1077,6 +1088,7 @@ feed_items_get(struct feed *f, FILE *fp, struct items *itemsret)
 	size_t cap, i, linesize = 0, nitems;
 	ssize_t linelen;
 	off_t offset;
+	int ret = -1;
 
 	cap = nitems = 0;
 	offset = 0;
@@ -1115,17 +1127,18 @@ feed_items_get(struct feed *f, FILE *fp, struct items *itemsret)
 		if (linelen <= 0 || feof(fp))
 			break;
 	}
-	free(line);
+	ret = 0;
+
+err:
 	itemsret->cap = cap;
 	itemsret->items = items;
 	itemsret->len = nitems;
-	return 0;
-
-err:
 	free(line);
-	feed_items_free(itemsret);
 
-	return -1;
+	if (ret)
+		feed_items_free(itemsret);
+
+	return ret;
 }
 
 void
@@ -1142,7 +1155,7 @@ updatenewitems(struct feed *f)
 		row = &(p->rows[i]); /* do not use pane_row_get */
 		item = (struct item *)row->data;
 		if (urlfile)
-			item->isnew = isurlnew(item->link);
+			item->isnew = urls_isnew(item->link);
 		else
 			item->isnew = (item->timeok && item->timestamp >= comparetime);
 		row->bold = item->isnew;
@@ -1191,7 +1204,7 @@ feed_count(struct feed *f, FILE *fp)
 		parseline(line, fields);
 
 		if (urlfile) {
-			f->totalnew += isurlnew(fields[FieldLink]);
+			f->totalnew += urls_isnew(fields[FieldLink]);
 		} else {
 			parsedtime = 0;
 			if (!strtotime(fields[FieldUnixTimestamp], &parsedtime))
@@ -1281,9 +1294,10 @@ feeds_reloadall(void)
 	off_t pos;
 
 	pos = panes[PaneItems].pos; /* store numeric position */
-	readurls();
 	feeds_set(curfeed); /* close and reopen feed if possible */
+	urls_read();
 	feeds_load(feeds, nfeeds);
+	urls_free();
 	/* restore numeric position */
 	pane_setpos(&panes[PaneItems], pos);
 	updatesidebar(onlynew);
@@ -1362,7 +1376,9 @@ sighandler(int signo)
 	case SIGINT:
 	case SIGTERM:
 	case SIGWINCH:
-		sigstate = signo;
+		/* SIGTERM is more important, do not override it */
+		if (sigstate != SIGTERM)
+			sigstate = signo;
 		break;
 	}
 }
@@ -1446,12 +1462,13 @@ mousereport(int button, int release, int x, int y)
 				break;
 			pane_setpos(p, pos);
 			if (selpane == PaneFeeds) {
-				readurls();
 				row = pane_row_get(p, p->pos);
 				f = (struct feed *)row->data;
 				feeds_set(f);
+				urls_read();
 				if (f->fp)
 					feed_load(f, f->fp);
+				urls_free();
 				/* redraw row: counts could be changed */
 				updatesidebar(onlynew);
 				updatetitle();
@@ -1638,24 +1655,36 @@ markread(struct pane *p, off_t from, off_t to, int isread)
 }
 
 int
-urlcmp(const void *v1, const void *v2)
+urls_cmp(const void *v1, const void *v2)
 {
 	return strcmp(*((char **)v1), *((char **)v2));
 }
 
+int
+urls_isnew(const char *url)
+{
+	return bsearch(&url, urls, nurls, sizeof(char *), urls_cmp) == NULL;
+}
+
 void
-readurls(void)
+urls_free(void)
+{
+	while (nurls > 0)
+		free(urls[--nurls]);
+	free(urls);
+	urls = NULL;
+	nurls = 0;
+}
+
+void
+urls_read(void)
 {
 	FILE *fp;
 	char *line = NULL;
 	size_t linesiz = 0, cap = 0;
 	ssize_t n;
 
-	while (nurls > 0)
-		free(urls[--nurls]);
-	free(urls);
-	urls = NULL;
-	nurls = 0;
+	urls_free();
 
 	if (!urlfile || !(fp = fopen(urlfile, "rb")))
 		return;
@@ -1672,13 +1701,7 @@ readurls(void)
 	fclose(fp);
 	free(line);
 
-	qsort(urls, nurls, sizeof(char *), urlcmp);
-}
-
-int
-isurlnew(const char *url)
-{
-	return bsearch(&url, urls, nurls, sizeof(char *), urlcmp) == NULL;
+	qsort(urls, nurls, sizeof(char *), urls_cmp);
 }
 
 int
@@ -1708,9 +1731,9 @@ main(int argc, char *argv[])
 	if ((tmp = getenv("SFEED_YANKER")))
 		yankercmd = tmp;
 	if ((tmp = getenv("SFEED_MARK_READ")))
-		markreadcmd = "sfeed_markread read";
+		markreadcmd = tmp;
 	if ((tmp = getenv("SFEED_MARK_UNREAD")))
-		markunreadcmd = "sfeed_markread unread";
+		markunreadcmd = tmp;
 	urlfile = getenv("SFEED_URL_FILE");
 
 	panes[PaneFeeds].row_format = feed_row_format;
@@ -1736,9 +1759,10 @@ main(int argc, char *argv[])
 		}
 		nfeeds = argc - 1;
 	}
-	readurls();
 	feeds_set(&feeds[0]);
+	urls_read();
 	feeds_load(feeds, nfeeds);
+	urls_free();
 
 	if (!isatty(0)) {
 		if ((fd = open("/dev/tty", O_RDONLY)) == -1)
@@ -1940,12 +1964,13 @@ nextpage:
 		case '\n':
 			p = &panes[selpane];
 			if (selpane == PaneFeeds && panes[selpane].nrows) {
-				readurls();
 				row = pane_row_get(p, p->pos);
 				f = (struct feed *)row->data;
 				feeds_set(f);
+				urls_read();
 				if (f->fp)
 					feed_load(f, f->fp);
+				urls_free();
 				/* redraw row: counts could be changed */
 				updatesidebar(onlynew);
 				updatetitle();
@@ -1998,7 +2023,6 @@ event:
 		else if (ch == -3 && sigstate == 0)
 			continue; /* just a time-out, nothing to do */
 
-		/* handle last signal */
 		switch (sigstate) {
 		case SIGHUP:
 			feeds_reloadall();
